package ksource.eclipse.util.serialver.actions;

import ksource.eclipse.util.ClassHelper;
import ksource.eclipse.util.serialver.SerialVersionBuilder;

import org.eclipse.core.resources.IFile;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.OpenFileAction;

/**
 * @author elvis
 */
public class AddSerialVersionAction implements IObjectActionDelegate {

    ISelection selection = null;
    
    IWorkbenchPart part = null;

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction,
     *      org.eclipse.ui.IWorkbenchPart)
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
        this.part = targetPart;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
     */
    public void run(IAction action) {
        if (selection instanceof IStructuredSelection) {
            IStructuredSelection struSelection = (IStructuredSelection) selection;
            
            Object firstElement = struSelection.getFirstElement();
            
            if(null != firstElement){
                IType type = null;
                if(firstElement instanceof IType){
                    type = (IType)firstElement;
                }else if(firstElement instanceof IFile){
                    type = getTypeOfUnit(JavaCore.createCompilationUnitFrom((IFile)firstElement));
                }else if(firstElement instanceof ICompilationUnit){
                    type = getTypeOfUnit((ICompilationUnit)firstElement);
                }
                if(null != type){
                    generateSerialVersion(type);
                }
            }
        }
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction,
     *      org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IAction action, ISelection selection) {
        this.selection = selection;
    }
    
    private void generateSerialVersion(IType type){
        try {
            Class clazz = null;
            
            try {
                clazz = ClassHelper.typeToClass(type);
            } catch (Exception e) {
                e.printStackTrace();
            }
            
            if (null != clazz) {
                long versionId = SerialVersionBuilder.buildSerialVersion(clazz);
                if (versionId != 0L) {
                    IField field = type.getField("serialVersionUID");
                    if(null != field && field.exists()){
                        field.delete(true,null);
                    }
                    String fieldContent = "\tstatic final long serialVersionUID = "
                            + versionId + "L;\n\n";
                    type.createField(fieldContent, getInsertPosition(type), false, null);
                    
                    OpenFileAction openAction = new OpenFileAction(part.getSite().getPage());
                    openAction.selectionChanged((IStructuredSelection)selection);
                    openAction.run();
                }
            }

        } catch (JavaModelException e) {
            e.printStackTrace();
        }        
    }
    
    private IType getTypeOfUnit(ICompilationUnit unit){
        IType type = null;
        if(null != unit){
            try {
                IType[] types = unit.getTypes();
                if(null != types && types.length > 0){
                    type = types[0];
                }
            } catch (JavaModelException e) {
                e.printStackTrace();
            }
        }        
        return type;
    }
    
    private IJavaElement getInsertPosition(IType type){
        IJavaElement element = null;
        
        try {
            IField[] fields = type.getFields();
            if(fields.length > 0){
                element = fields[0];
            }
        } catch (JavaModelException e) {
            e.printStackTrace();
        }
        
        return element;
    }

}